@@ -4,18 +4,23 @@
 # Copyright (c) 2017-2021 OWS Contributors
 # SPDX-License-Identifier: Apache-2.0
 import json
+import logging
 import os
 from importlib import import_module
-from typing import (Any, Iterable, List, Mapping, MutableMapping, Optional,
-                    Sequence, Set, Union, cast)
+from typing import (Any, Callable, Iterable, List, Mapping, MutableMapping,
+                    Optional, Sequence, Set, Union, cast)
 from urllib.parse import urlparse
 
 import fsspec
+from datacube.utils.masking import make_mask
 from flask_babel import gettext as _
+from xarray import DataArray
 
 from datacube_ows.config_toolkit import deepinherit
 from datacube_ows.ogc_utils import ConfigException, FunctionWrapper
 
+_LOG = logging.getLogger(__name__)
+
 RAW_CFG = Union[
         None,
         str,
@@ -99,6 +104,8 @@ def get_file_loc(x: str) -> str:
             raise ConfigException("Please set environment variable 'DATACUBE_OWS_CFG_ALLOW_S3=YES' "
                               + "to enable OWS config from AWS S3")
         cwd = xp.scheme + "://" + xp.netloc +  xp.path.rsplit("/", 1)[0]
+    elif xp.scheme:
+        raise ConfigException(f"Unsupported URL scheme in config inheritance: {xp.scheme}")
     else:
         abs_path = os.path.abspath(x)
         cwd = os.path.dirname(abs_path)
@@ -764,12 +771,12 @@ class FlagProductBands(OWSConfigEntry):
         for mask in masks:
             handled = False
             for fp in flag_products:
-                if fp.products_match(mask.band.pq_names):
-                    fp.add_flag_band(mask.band)
+                if fp.products_match(mask.flag_band.pq_names):
+                    fp.add_flag_band(mask.flag_band)
                     handled = True
                     break
             if not handled:
-                flag_products.append(cls(mask.band, layer))
+                flag_products.append(cls(mask.flag_band, layer))
         return flag_products
 
     @classmethod
@@ -795,3 +802,87 @@ class FlagProductBands(OWSConfigEntry):
             if not handled:
                 flag_products.append(cls(fb, layer))
         return flag_products
+
+
+class AbstractMaskRule(OWSConfigEntry):
+    def __init__(self, band: str, cfg: CFG_DICT, mapper: Callable[[str], str] = lambda x: x) -> None:
+        super().__init__(cfg)
+        self.band = mapper(band)
+        self.parse_rule_spec(cfg)
+
+    @property
+    def context(self) -> str:
+        return "a mask rule"
+
+    VALUES_LABEL = "values"
+    def parse_rule_spec(self, cfg: CFG_DICT):
+        self.flags: Optional[CFG_DICT] = None
+        self.or_flags: bool = False
+        self.values: Optional[List[int]] = None
+        self.invert: bool = cfg.get("invert", False)
+        if "flags" in cfg:
+            flags = cast(CFG_DICT, cfg["flags"])
+            self.or_flags: bool = False
+            if "or" in flags and "and" in flags:
+                raise ConfigException(
+                    f"ValueMap rule in {self.context} combines 'and' and 'or' rules")
+            elif "or" in flags:
+                self.or_flags = True
+                flags = cast(CFG_DICT, flags["or"])
+            elif "and" in flags:
+                flags = cast(CFG_DICT, flags["and"])
+            self.flags: Optional[CFG_DICT] = flags
+        else:
+            self.flags = None
+            self.or_flags = False
+
+        if "values" in cfg:
+            val: Any = cfg["values"]
+        elif "enum" in cfg:
+            val = cfg["enum"]
+            _LOG.warning("enum in pq_masks is deprecated and will be removed in a future release. Refer to the documentation for the new syntax.")
+        else:
+            val = None
+        if val is None:
+            self.values = None
+        else:
+            if isinstance(val, int):
+                self.values: Optional[List[int]] = [cast(int, val)]
+            else:
+                self.values: Optional[List[int]] = cast(List[int], val)
+
+        if not self.flags and not self.values:
+            raise ConfigException(
+                f"Mask rule in {self.context} must have a non-empty 'flags' or 'values' section.")
+        if self.flags and self.values:
+            raise ConfigException(
+                f"Mask rule in {self.context} has both a 'flags' and a 'values' section - choose one.")
+
+    def create_mask(self, data: DataArray) -> DataArray:
+        """
+        Create a mask from raw flag band data.
+
+        :param data: Raw flag data, assumed to be for this rule's flag band.
+        :return: A boolean DataArray, True where the data matches this rule
+        """
+        if self.values:
+            mask: Optional[DataArray] = None
+            for v in cast(List[int], self.values):
+                vmask = data == v
+                if mask is None:
+                    mask = vmask
+                else:
+                    mask |= vmask
+        elif self.or_flags:
+            mask = None
+            for f in cast(CFG_DICT, self.flags).items():
+                f = {f[0]: f[1]}
+                if mask is None:
+                    mask = make_mask(data, **f)
+                else:
+                    mask |= make_mask(data, **f)
+        else:
+            mask = make_mask(data, **cast(CFG_DICT, self.flags))
+        if mask is not None and self.invert:
+            mask = ~mask # pylint: disable=invalid-unary-operand-type
+        return mask
