@@ -3,47 +3,65 @@
 #
 # Copyright (c) 2017-2021 OWS Contributors
 # SPDX-License-Identifier: Apache-2.0
+from typing import (Any, Callable, Hashable, List, MutableMapping, Optional,
+                    Union, cast)
+
 import numpy as np
 from xarray import DataArray, Dataset
 
+from datacube_ows.config_utils import CFG_DICT
 from datacube_ows.ogc_utils import ConfigException, FunctionWrapper
 from datacube_ows.styles.base import StyleDefBase
 
 # pylint: disable=abstract-method
 
+LINEAR_COMP_DICT = MutableMapping[str, Union[float, List[float]]]
+
 
 class ComponentStyleDef(StyleDefBase):
-    def __init__(self, product, style_cfg,
-                 stand_alone=False, defer_multi_date=False, user_defined=False):
+    """
+    Style Subclass that allows the behaviour of each component (red, green, blue, alpha) to be
+    specified independently.
+    """
+    def __init__(self, product: "datacube_ows.ows_configuration.OWSNamedLayer",
+                 style_cfg: CFG_DICT,
+                 stand_alone: bool = False,
+                 defer_multi_date: bool = False,
+                 user_defined: bool = False) -> None:
+        """
+        See superclass
+        """
         super().__init__(product, style_cfg,
                          stand_alone=stand_alone, defer_multi_date=defer_multi_date, user_defined=user_defined)
-        style_cfg = self._raw_cfg
-        self.raw_rgb_components = {}
+        style_cfg: CFG_DICT = cast(CFG_DICT, self._raw_cfg)
+        self.raw_rgb_components: MutableMapping[str, Union[Callable, LINEAR_COMP_DICT]] = {}
+        raw_components = cast(MutableMapping[str, Union[Callable, CFG_DICT]], style_cfg["components"])
         for imgband in ["red", "green", "blue", "alpha"]:
-            components = style_cfg["components"].get(imgband)
+            components = raw_components.get(imgband)
             if components is None:
                 if imgband == "alpha":
                     continue
                 else:
                     raise ConfigException(f"No components defined for {imgband} band in style {self.name}, layer {product.name}")
-            if callable(components) or "function" in components:
+            elif callable(components) or "function" in components:
                 self.raw_rgb_components[imgband] = FunctionWrapper(self.product, components,
                                                                    stand_alone=self.stand_alone)
                 if not self.stand_alone:
                     if "additional_bands" not in style_cfg:
                         raise ConfigException(f"Style with a function component must declare additional_bands.")
-                    for b in style_cfg.get("additional_bands", set()):
+                    for b in cast(List[str], style_cfg.get("additional_bands", [])):
                         self.raw_needed_bands.add(b)
             else:
+                components = cast(LINEAR_COMP_DICT, components)
                 self.raw_rgb_components[imgband] = components
                 for k in components.keys():
                     if k != "scale_range":
                         self.raw_needed_bands.add(k)
-        self.declare_unready("rgb_components")
+        self.rgb_components = cast(MutableMapping[str, Union[None, Callable, LINEAR_COMP_DICT]], {})
 
         self.scale_factor = style_cfg.get("scale_factor")
         if "scale_range" in style_cfg:
-            self.scale_min, self.scale_max = style_cfg["scale_range"]
+            self.scale_min, self.scale_max = cast(List[Optional[float]], style_cfg["scale_range"])
         elif self.scale_factor:
             self.scale_min = 0.0
             self.scale_max = 255.0 * self.scale_factor
@@ -51,12 +69,13 @@ class ComponentStyleDef(StyleDefBase):
             self.scale_min = None
             self.scale_max = None
 
-        self.component_scale_ranges = {}
-        for cn, cd in style_cfg["components"].items():
+        self.component_scale_ranges: MutableMapping[str, MutableMapping[str, float]] = {}
+        for cn, cd in raw_components.items():
             if not callable(cd) and "scale_range" in cd:
+                scale_range = cast(List[float], cd["scale_range"])
                 self.component_scale_ranges[cn] = {
-                    "min": cd["scale_range"][0],
-                    "max": cd["scale_range"][1],
+                    "min": scale_range[0],
+                    "max": scale_range[1],
                 }
             else:
                 self.component_scale_ranges[cn] = {
@@ -65,8 +84,15 @@ class ComponentStyleDef(StyleDefBase):
                 }
 
     # pylint: disable=attribute-defined-outside-init
-    def make_ready(self, dc, *args, **kwargs):
-        self.rgb_components = {}
+    def make_ready(self, dc, *args, **kwargs) -> None:
+        """
+        Second-phase (db aware) initialisation
+
+        Mostly sorting out bands, esp flag bands.
+
+        :param dc: A datacube object
+        """
+        self.rgb_components = cast(MutableMapping[str, Union[None, Callable, LINEAR_COMP_DICT]], {})
         for band, component in self.raw_rgb_components.items():
             if not component or callable(component):
                 self.rgb_components[band] = component
@@ -74,28 +100,52 @@ class ComponentStyleDef(StyleDefBase):
                 self.rgb_components[band] = self.dealias_components(component)
         super().make_ready(dc, *args, **kwargs)
 
-    def dealias_components(self, comp_in):
+
+        self.raw_rgb_components: MutableMapping[str, Union[Callable, LINEAR_COMP_DICT]] = {}
+
+    def dealias_components(self, comp_in: Optional[LINEAR_COMP_DICT]) -> Optional[LINEAR_COMP_DICT]:
+        """
+        Convert a component dictionary with band aliases to a component dictionary using canonical band names.
+
+        :param comp_in: A linear component dictionary with band aliases
+        :return: Equivalent linear component dictionary with canonical band names.
+        """
         if self.stand_alone:
             return comp_in
         elif comp_in is None:
             return None
         else:
-            return {self.product.band_idx.band(self.local_band(band_alias)): value for band_alias, value in comp_in.items() if band_alias not in ['scale_range']}
+            return {
+                self.product.band_idx.band(self.local_band(band_alias)): value
+                for band_alias, value in comp_in.items() if band_alias not in ['scale_range']
+            }
 
-    def compress_band(self, component_name, imgband_data):
-        sc_min = self.component_scale_ranges[component_name]["min"]
-        sc_max = self.component_scale_ranges[component_name]["max"]
+    def compress_band(self, component_name: str, imgband_data: "xarray.DataArray") -> "xarray.DataArray":
+        """
+        Compress dynamic range of a component data array to uint8 range (0-255)
+
+        :param component_name: The name of the component being compressed (i.e. 'red', 'green', 'blue' or 'alpha')
+        :param imgband_data: The input, uncompressed DataArray for the component
+        :return:
+        """
+        sc_min: float = self.component_scale_ranges[component_name]["min"]
+        sc_max: float = self.component_scale_ranges[component_name]["max"]
         clipped = imgband_data.clip(sc_min, sc_max)
         normalized = (clipped - sc_min) / (sc_max - sc_min)
         return normalized * 255
 
 
-    def transform_single_date_data(self, data):
-        imgdata = {}
+    def transform_single_date_data(self, data: "xarray.Dataset") -> "xarray.Dataset":
+        """
+        Apply style to raw data to make an RGBA image xarray (single time slice only)
+
+        :param data: Raw data, all bands.
+        :return: RGBA uint8 xarray
+        """
+        imgdata = cast(MutableMapping[Hashable, Any], {})
         for imgband, components in self.rgb_components.items():
             if callable(components):
                 imgband_data = components(data)
-                dims = imgband_data.dims
                 imgband_data = imgband_data.astype('uint8')
                 imgdata[imgband] = imgband_data
             else:
@@ -113,8 +163,8 @@ class ComponentStyleDef(StyleDefBase):
                     else:
                         imgband_data = imgband_component
                 if imgband_data is None:
-                    imgband_data = np.zeros(list(data.dims.values()), 'uint8')
-                    imgband_data = DataArray(imgband_data, data.coords, data.dims.keys())
+                    null_np = np.zeros(list(data.dims.values()), 'uint8')
+                    imgband_data = DataArray(null_np, data.coords, data.dims.keys())
                 if imgband != "alpha":
                     imgband_data = self.compress_band(imgband, imgband_data)
                 imgdata[imgband] = imgband_data.astype("uint8")
@@ -123,4 +173,5 @@ class ComponentStyleDef(StyleDefBase):
         return image_dataset
 
 
+# Register ComponentStyleDef as a Style subclass.
 StyleDefBase.register_subclass(ComponentStyleDef, "components")
