@@ -16,7 +16,12 @@ import numpy
 from colour import Color
 from matplotlib import pyplot as plt
 from matplotlib.colors import LinearSegmentedColormap, to_hex
-from numpy.typing import NDArray
+
+try:
+    from numpy.typing import NDArray
+except ImportError:
+    NDArray = numpy.ndarray
+
 from xarray import Dataset
 
 from datacube_ows.config_utils import CFG_DICT
@@ -563,11 +568,6 @@ class ColorRampDef(StyleDefBase):
                            )
 
     class MultiDateHandler(StyleDefBase.MultiDateHandler):
-        """
-        MultiDateHandler base class.
-
-        Handles "aggregator"-based index-value multi-date requests (e.g. delta)
-        """
         auto_legend = True
 
         def __init__(self, style: "ColorRampDef", cfg: CFG_DICT) -> None:
